@@ -110,7 +110,7 @@ TODO: Prevent crash if expr_src is unavailable when tracing...
 
 # TODO: Cache compiled ASTs!
 
-__version__ = "0.13"
+__version__ = "0.14"
 
 import sys
 import inspect
@@ -184,6 +184,14 @@ COLORS = True
 Whether to print ANSI color control sequences to color the output or not.
 """
 
+IGNORE_TRAILING_WHITESPACE = True
+"""
+Controls equality and inclusion tests on strings, including multiline
+strings, causing them to ignore trailing whitespace. True by default,
+since trailing whitespace is hard to reason about because it's
+invisible.
+"""
+
 
 #--------#
 # Errors #
@@ -798,7 +806,7 @@ def check_expectation(exp, set_success=True):
         short_expected = ellipsis(repr(expected))
         full_result = repr(result)
         full_expected = repr(expected)
-        if result == expected:
+        if checkEquality(result, expected):
             msg = create_success_message(
                 exp["tag"],
                 context,
@@ -864,7 +872,7 @@ def check_expectation(exp, set_success=True):
         short_fragment = ellipsis(fragment)
         full_output = output
         full_fragment = fragment
-        if fragment in output:
+        if checkContainment(fragment, output):
             msg = create_success_message(
                 exp["tag"],
                 context,
@@ -1061,9 +1069,50 @@ def expectCustom(tester):
         return check_expectation(exp)
 
 
-#-------------------#
-# Verbosity control #
-#-------------------#
+def checkEquality(val1, val2):
+    """
+    Returns True if val1 is 'equal' to val2, and False otherwise.
+    If IGNORE_TRAILING_WHITESPACE is True, will ignore trailing
+    whitespace in two strings when comparing them for equality.
+    """
+    if (not isinstance(val1, str)) or (not isinstance(val2, str)):
+        return val1 == val2 # use regular equality test
+    # For two strings, pay attention to IGNORE_TRAILING_WHITESPACE
+    elif IGNORE_TRAILING_WHITESPACE:
+        # remove trailing whitespace from both strings (on all lines)
+        return trimWhitespace(val1) == trimWhitespace(val2)
+    else:
+        return val1 == val2 # use regular equality test
+
+
+def checkContainment(val1, val2):
+    """
+    Returns True if val1 is 'contained in' to val2, and False otherwise.
+    If IGNORE_TRAILING_WHITESPACE is True, will ignore trailing
+    whitespace in two strings when comparing them for containment.
+    """
+    if (not isinstance(val1, str)) or (not isinstance(val2, str)):
+        return val1 in val2 # use regular containment test
+    # For two strings, pay attention to IGNORE_TRAILING_WHITESPACE
+    elif IGNORE_TRAILING_WHITESPACE:
+        # remove trailing whitespace from both strings (on all lines)
+        return trimWhitespace(val1) in trimWhitespace(val2)
+    else:
+        return val1 in val2 # use regular containment test
+
+
+def trimWhitespace(st):
+    """
+    Assume st a string. Use .rstrip() to remove trailing whitespace from
+    each line. This has the side effect of replacing complex newlines
+    with just '\\n'.
+    """
+    return '\n'.join(line.rstrip() for line in st.splitlines())
+
+
+#-----------------------#
+# Configuration control #
+#-----------------------#
 
 def detailLevel(level):
     """
@@ -1080,6 +1129,19 @@ def detailLevel(level):
     DETAIL_LEVEL = level
 
 
+def attendTrailingWhitespace(on=True):
+    """
+    Call this function to force `optimism` to pay attention to
+    whitespace at the end of lines when checking expectations. By
+    default, such whitespace is removed both from expected
+    values/output fragments and from captured outputs/results before
+    checking expectations. To turn that functionality on again, you
+    can call this function with False as the argument.
+    """
+    global IGNORE_TRAILING_WHITESPACE
+    IGNORE_TRAILING_WHITESPACE = not on
+
+
 #---------------#
 # Summarization #
 #---------------#
